#version 430 core

in vec2 texCoords;

out vec4 color;

layout(binding = 0) uniform sampler2D screenTexture;
layout(binding = 1) uniform sampler2D depthTexture;
uniform mat4 invViewProjection;
uniform mat4 prevViewProjection;

void main() {
	vec2 texCoord = texCoords;
	float depth = texture(depthTexture, texCoord).r;
	vec4 H = vec4(texCoord.x * 2 - 1, (1 - texCoord.y) * 2 - 1, depth, 1);
	vec4 D = invViewProjection * H;
	vec4 worldPos = D / D.w;

	vec4 currPos = H;
	vec4 prevPos = prevViewProjection * worldPos;
	prevPos /= prevPos.w;
	vec2 pixelVelocity = (currPos - prevPos).xy / 8.f;

	// In the article this is set to texture(screenTexture, texCoord), which falsifies colors in our case
	vec4 currColor = vec4(0.0, 0.0, 0.0, 0.0);
	texCoord += pixelVelocity;
	int samples = 8;
	for (int i = 0; i < samples; ++i, texCoord += pixelVelocity) {
		vec4 sampleColor = texture(screenTexture, texCoord);
		currColor += sampleColor;
	}

	color = currColor / samples;
}